# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2006 Lukáš Lalinský
# Copyright (C) 2010 Petr Viktorin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtGui
from picard.plugin import ExtensionPoint


class OptionsCheckError(Exception):

    def __init__(self, title, info):
        self.title = title
        self.info = info

class OptionsPage(QtGui.QWidget):

    PARENT = None
    SORT_ORDER = 1000
    ACTIVE = True

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui_objects = {}
        if not hasattr(self, 'AUTO_UI'):
            print "Warning - this OptionsPage has no attribute AUTO_UI:", self.__class__.__name__
            return
        if self.AUTO_UI:
            self.setupUi()

    def info(self):
        raise NotImplementedError

    def check(self):
        pass

    def load(self):
        for name, item in self.ui_objects.items():
            item.setValue(self.config.setting[name])

    def save(self):
        for name, item in self.ui_objects.items():
            self.config.setting[name] = item.getValue()

    def display_error(self, error):
        dialog = QtGui.QMessageBox(QtGui.QMessageBox.Warning, error.title, error.info, QtGui.QMessageBox.Ok, self)
        dialog.exec_()

    def setupUi(self):
        self.setObjectName(self.__class__.__name__)
        vboxlayout = QtGui.QVBoxLayout(self)
        self.resize(463, 388)
        vboxlayout.setSpacing(6)
        vboxlayout.setMargin(9)
        sections = {}
        self.ui_objects = {}
        for option in self.options:
            try:
                parameters = dict(option.ui_parameters)
            except TypeError:
                parameters = {}
            if parameters is None:
                parameters = dict()
            cls = parameters.pop('cls', option.ui_class)
            section = parameters.pop('section', None)
            item = cls(option, self, parameters)
            if section:
                try:
                    target_layout = sections[section]
                except KeyError:
                    if isinstance(self.sections[section], basestring):
                        section_params = dict(caption=self.sections[section])
                    else:
                        section_params = self.sections[section]
                    if section_params.get('checkable', False):
                        groupbox = item.widget
                    else:
                        groupbox = QtGui.QGroupBox(self)
                    try:
                        groupbox.setTitle(_(section_params['caption']))
                    except KeyError:
                        groupbox.setTitle(section)
                    vboxlayout.addWidget(groupbox)
                    if section_params.get('horizontal', False):
                        target_layout = QtGui.QHBoxLayout(groupbox)
                    else:
                        target_layout = QtGui.QVBoxLayout(groupbox)
                    sections[section] = target_layout
            else:
                target_layout = vboxlayout
            item.addToLayout(target_layout)
            self.ui_objects[option.name] = (item)
        vboxlayout.addItem(QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding))

_pages = ExtensionPoint()

def register_options_page(page_class):
    _pages.register(page_class.__module__, page_class)
