# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2006 Lukáš Lalinský
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.config import BoolOption, TextOption
from picard.ui.options import OptionsPage, register_options_page
from picard.ui.option import Section

# gettext is only initialized after the import; work around that
def _(x):
    return x

class CoverOptionsPage(OptionsPage):

    NAME = "cover"
    TITLE = N_("Cover Art")
    PARENT = None
    SORT_ORDER = 35
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        ratings=dict(caption=_("Save cover images as separate files"), checkable=True),
    )

    options = [
        BoolOption("setting", "save_images_to_tags", True,
            ui=dict(
                caption=_("Embed cover images into tags"),
            )),
        BoolOption("setting", "save_images_to_files", False,
            ui=dict(
                cls=Section,
                caption=_("Save cover images as separate files"),
                section="ratings",
            )),
        TextOption("setting", "cover_image_filename", "cover",
            ui=dict(
                caption=_("Cover image filename (without the extension)"),
                section="ratings",
            )),
        BoolOption("setting", "save_images_overwrite", False,
            ui=dict(
                caption=_("Overwrite the file if it already exists"),
                section="ratings",
            )),
    ]

register_options_page(CoverOptionsPage)
