# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2008 Lukáš Lalinský
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.config import BoolOption, TextOption, IntOption
from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
from picard.ui.option import ComboBox

# gettext is only initialized after the import; work around that
def _(x):
    return x

class FolksonomyOptionsPage(OptionsPage):

    NAME = "folsonomy"
    TITLE = N_("Folksonomy Tags")
    PARENT = "metadata"
    SORT_ORDER = 20
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        folksonomy=_("Folksonomy Tags"),
    )

    options = [
        TextOption("setting", "ignore_tags", "seen live,favorites,fixme,owned",
            ui=dict(
                caption=_("Tags to ignore"),
                default_button=True,
                section="folksonomy",
            )),
        BoolOption("setting", "only_my_tags", False,
            ui=dict(
                caption=_("Only use my tags"),
                section="folksonomy",
            )),
        IntOption("setting", "min_tag_usage", 90,
            ui=dict(
                caption=_("Minimal tag usage"),
                section="folksonomy",
                suffix=" %",
            )),
        IntOption("setting", "max_tags", 5,
            ui=dict(
                caption=_("Maximum number of tags"),
                section="folksonomy",
            )),
        TextOption("setting", "join_tags", "",
            ui=dict(
                cls=ComboBox,
                caption=_("Join multiple tags with"),
                section="folksonomy",
                values=[" / ", ", "],
                horizontal=True,
            )),
    ]

register_options_page(FolksonomyOptionsPage)
