# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2006 Lukáš Lalinský
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from picard.config import IntOption, TextOption, BoolOption, PasswordOption
from picard.ui.options import OptionsPage, register_options_page
from picard.ui.option import ComboBox

# gettext is only initialized after the import; work around that
def _(x):
    return x

class GeneralOptionsPage(OptionsPage):

    NAME = "general"
    TITLE = N_("General")
    PARENT = None
    SORT_ORDER = 1
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        server=dict(caption=_("MusicBrainz Server"), horizontal=True),
        account=_("Account Information"),
        general=_("General"),
    )

    options = [
        TextOption("setting", "server_host", "musicbrainz.org",
            ui=dict(
                cls=ComboBox,
                section='server',
                caption=_("Server address"),
                stretch=1,
            )),
        IntOption("setting", "server_port", 80,
            ui=dict(
                section='server',
                caption=_("Port"),
                horizontal=False,
            )),
        TextOption("setting", "username", "",
            ui=dict(
                section='account',
                caption=_("Username"),
            )),
        PasswordOption("setting", "password", "",
            ui=dict(
                section='account',
                caption=_("Password"),
            )),
        BoolOption("setting", "analyze_new_files", False,
            ui=dict(
                section='general',
                caption=_("Automatically scan all new files"),
            )),
    ]

    def __init__(self, parent=None):
        super(GeneralOptionsPage, self).__init__(parent)
        mirror_servers = [
            "musicbrainz.org",
            "nl.musicbrainz.org",
            "de.musicbrainz.org",
            ]
        self.ui_objects['server_host'].widget.addItems(sorted(mirror_servers))


register_options_page(GeneralOptionsPage)
