# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2006 Lukáš Lalinský
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.config import BoolOption, TextOption
from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
from picard.const import RELEASE_COUNTRIES
import operator
import locale
from picard.ui.option import ComboBox

# gettext is only initialized after the import; work around that
def _(x):
    return x

class MetadataOptionsPage(OptionsPage):

    NAME = "metadata"
    TITLE = N_("Metadata")
    PARENT = None
    SORT_ORDER = 20
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        metadata=_("Metadata"),
        custom=_("Custom Fields"),
    )

    options = [
        BoolOption("setting", "translate_artist_names", False,
            ui=dict(
                caption=_("Translate foreign artist names to English where possible"),
                section='metadata',
            )),
        BoolOption("setting", "release_ars", True,
            ui=dict(
                caption=_("Use release relationships"),
                section='metadata',
            )),
        BoolOption("setting", "track_ars", False,
            ui=dict(
                caption=_("Use track relationships"),
                section='metadata',
            )),
        BoolOption("setting", "folksonomy_tags", False,
            ui=dict(
                caption=_("Use folksonomy tags as genre"),
                section='metadata',
            )),
        TextOption("setting", "preferred_release_country", u"",
            ui=dict(
                cls=ComboBox,
                editable=False,
                caption=_("Preferred release country:"),
                section='metadata',
            )),
        TextOption("setting", "va_name", u"Various Artists",
            ui=dict(
                caption=_("Various artists"),
                section='custom',
                default_button=True,
            )),
        TextOption("setting", "nat_name", u"[non-album tracks]",
            ui=dict(
                caption=_("Non-album tracks"),
                section='custom',
                default_button=True,
            )),
    ]

    def __init__(self, parent=None):
        super(MetadataOptionsPage, self).__init__(parent)
        self.ui_objects['preferred_release_country'].widget.addItem(_("None"), QtCore.QVariant(""))
        country_list = [(c[0], _(c[1])) for c in RELEASE_COUNTRIES.items()]
        for country, name in sorted(country_list, key=operator.itemgetter(1),
                                    cmp=locale.strcoll):
            self.ui_objects['preferred_release_country'].widget.addItem(name, QtCore.QVariant(country))

register_options_page(MetadataOptionsPage)
