# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2008 Philipp Wolfer
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.config import BoolOption, TextOption, IntOption
from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
from picard.ui.option import Section, NoUi
import re

# gettext is only initialized after the import; work around that
def _(x):
    return x

_helptext=_("Picard saves the ratings together with an e-mail address identifying the user who did the rating. That way different ratings for different users can be stored in the files. Please specify the e-mail you want to use to save your ratings.")

class RatingsOptionsPage(OptionsPage):

    NAME = "ratings"
    TITLE = N_("Ratings")
    PARENT = "metadata"
    SORT_ORDER = 20
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        ratings=dict(caption=_("Enable track ratings"), checkable=True),
    )

    options = [
        BoolOption("setting", "enable_ratings", False,
            ui=dict(
                cls=Section,
                section="ratings",
                description=_helptext,
            )),
        TextOption("setting", "rating_user_email", "users@musicbrainz.org",
            ui=dict(
                caption=_("E-mail"),
                section="ratings",
            )),
        BoolOption("setting", "submit_ratings", True,
            ui=dict(
                caption=_("Submit ratings to MusicBrainz"),
                section="ratings",
            )),
        IntOption("setting", "rating_steps", 6,
            ui=dict(cls=NoUi)),
    ]

register_options_page(RatingsOptionsPage)
