# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2006 Lukáš Lalinský
# Copyright (C) 2010 Petr Viktorin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.config import BoolOption, TextOption
from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
from picard.ui.option import RadioBoxes, NoUi

# gettext is only initialized after the import; work around that
def _(x):
    return x

class TagsOptionsPage(OptionsPage):

    NAME = "tags"
    TITLE = N_("Tags")
    PARENT = None
    SORT_ORDER = 30
    ACTIVE = True
    AUTO_UI = True

    sections = dict(
        before=_("Before tagging"),
        compat=_("Tag compatibility"),
    )

    options = [
        BoolOption("setting", "dont_write_tags", False,
            ui=dict(
                inverse=True,
                caption=_("Write tags to files"),
            )),
        BoolOption("setting", "clear_existing_tags", False,
            ui=dict(
                caption=_("Clear existing tags"),
                section="before",
            )),
        BoolOption("setting", "remove_id3_from_flac", False,
            ui=dict(
                caption=_("Remove ID3 tags from FLAC files"),
                section="before",
            )),
        BoolOption("setting", "remove_ape_from_mp3", False,
            ui=dict(
                caption=_("Remove APE2 tags from MP3 files"),
                section="before",
            )),
        BoolOption("setting", "write_id3v23", False,
            ui=dict(
                cls=RadioBoxes,
                caption=_("ID3v2 version"),
                section="compat",
                values=[(False, _("2.2")), (True, _("2.3"))],
                callback="update_encodings",
            )),
        BoolOption("setting", "write_id3v1", True,
            ui=dict(
                caption=_("Also include ID3v1 tags in the files"),
                section="compat",
            )),
        TextOption("setting", "id3v2_encoding", "utf-8",
            ui=dict(
                cls=RadioBoxes,
                caption=_("ID3v2 text encoding"),
                section="compat",
                values=[('utf-8', _("UTF-8")), ('utf-16', _("UTF-16")), ('iso-8859-1', _("ISO-8859-1"))],
            )),
        BoolOption("setting", "tpe2_albumartist", False,
            ui=dict(cls=NoUi)),
    ]

    def load(self):
        super(TagsOptionsPage, self).load()
        self.update_encodings()

    def save(self):
        super(TagsOptionsPage, self).save()
        self.tagger.window.enable_tag_saving_action.setChecked(not self.config.setting["dont_write_tags"])

    def update_encodings(self):
        enc = self.ui_objects['id3v2_encoding']
        if not self.ui_objects['write_id3v23'].getValue():
            if enc.getValue() == 'utf-8':
                enc.setValue('utf-16')
            enc.setOptionEnabled('utf-8', False)
        else:
            enc.setOptionEnabled('utf-8', True)


register_options_page(TagsOptionsPage)
